"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .commercereportcomponent import (
    CommerceReportComponent,
    CommerceReportComponentTypedDict,
)
from .commercereportdimension import (
    CommerceReportDimension,
    CommerceReportDimensionTypedDict,
)
from .commercereporterror import CommerceReportError, CommerceReportErrorTypedDict
from .commercereportmeasure import CommerceReportMeasure, CommerceReportMeasureTypedDict
from codat_assess.types import BaseModel
import pydantic
from typing import Dict, List, Optional, TypedDict
from typing_extensions import Annotated, NotRequired


class ReportTypedDict(TypedDict):
    r"""## Structure

    Assess reports follow a consistent structure. Reports contain four sections of information:

    ### 1. Report definition

    Information such as:

    1. The report info (e.g. enhanced_profit_and_loss).
    2. The display name of the report (e.g. Enhanced Profit and Loss).

    ### 2. Dimension info

    Information about the dimension contained in the reports such as:

    1. The type of dimension (e.g. datetime, recordRef).
    2. The display name of the dimension (e.g. Period, Category type, Category sub type).
    3. The details about each item within the dimension (e.g. displayName:\"Jan 2022\", start:\"...\", end:\"...\", id:\"...\", name:\"...\").

    ### 3. Measure info

    Information about the measures contained in the report such as:

    1. The display name of the measure (e.g. value of account, percentage change).
    2. The type of the measure (e.g. currency, percentage).
    3. The unit of the measure (e.g. %, GBP).

    ### 4. The data for the report

    When the *includeDisplayName* parameter is set to *true*, it shows the *dimensionDisplayName* and *itemDisplayName* to make the data human-readable. The default setting for *includeDisplayName* is *false*.


    ## Displaying the report

    Reports can be rendered as follows (ordering is implicit rather than explicit):

    ![A table showing an example of how a report can be rendered](https://files.readme.io/1fa20ca-Report1.png)

    # Data model

    ## Dimensions
    """

    dimensions: NotRequired[List[CommerceReportDimensionTypedDict]]
    errors: NotRequired[List[CommerceReportErrorTypedDict]]
    measures: NotRequired[List[CommerceReportMeasureTypedDict]]
    report_data: NotRequired[List[CommerceReportComponentTypedDict]]
    report_info: NotRequired[Dict[str, str]]


class Report(BaseModel):
    r"""## Structure

    Assess reports follow a consistent structure. Reports contain four sections of information:

    ### 1. Report definition

    Information such as:

    1. The report info (e.g. enhanced_profit_and_loss).
    2. The display name of the report (e.g. Enhanced Profit and Loss).

    ### 2. Dimension info

    Information about the dimension contained in the reports such as:

    1. The type of dimension (e.g. datetime, recordRef).
    2. The display name of the dimension (e.g. Period, Category type, Category sub type).
    3. The details about each item within the dimension (e.g. displayName:\"Jan 2022\", start:\"...\", end:\"...\", id:\"...\", name:\"...\").

    ### 3. Measure info

    Information about the measures contained in the report such as:

    1. The display name of the measure (e.g. value of account, percentage change).
    2. The type of the measure (e.g. currency, percentage).
    3. The unit of the measure (e.g. %, GBP).

    ### 4. The data for the report

    When the *includeDisplayName* parameter is set to *true*, it shows the *dimensionDisplayName* and *itemDisplayName* to make the data human-readable. The default setting for *includeDisplayName* is *false*.


    ## Displaying the report

    Reports can be rendered as follows (ordering is implicit rather than explicit):

    ![A table showing an example of how a report can be rendered](https://files.readme.io/1fa20ca-Report1.png)

    # Data model

    ## Dimensions
    """

    dimensions: Optional[List[CommerceReportDimension]] = None

    errors: Optional[List[CommerceReportError]] = None

    measures: Optional[List[CommerceReportMeasure]] = None

    report_data: Annotated[
        Optional[List[CommerceReportComponent]], pydantic.Field(alias="reportData")
    ] = None

    report_info: Annotated[
        Optional[Dict[str, str]], pydantic.Field(alias="reportInfo")
    ] = None
